"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class RedisMultiCommand {
    queue = [];
    scriptsInUse = new Set();
    addCommand(args, transformReply) {
        this.queue.push({
            args,
            transformReply
        });
    }
    addScript(script, args, transformReply) {
        const redisArgs = [];
        redisArgs.preserve = args.preserve;
        if (this.scriptsInUse.has(script.SHA1)) {
            redisArgs.push('EVALSHA', script.SHA1);
        }
        else {
            this.scriptsInUse.add(script.SHA1);
            redisArgs.push('EVAL', script.SCRIPT);
        }
        if (script.NUMBER_OF_KEYS !== undefined) {
            redisArgs.push(script.NUMBER_OF_KEYS.toString());
        }
        redisArgs.push(...args);
        this.addCommand(redisArgs, transformReply);
    }
    transformReplies(rawReplies) {
        return rawReplies.map((reply, i) => {
            const { transformReply, args } = this.queue[i];
            return transformReply ? transformReply(reply, args.preserve) : reply;
        });
    }
}
exports.default = RedisMultiCommand;
